clear all; clc;

% -------------------------------------------------------------------------
% Define constants
% -------------------------------------------------------------------------
c0 = 299792458;                 % speed of light
me = 9.1e-31;                   % electron mass
e = 1.6e-19;                   % electron charge
a = 0;                          % assymetry parameter
gam = 0;                        % damping parameter
w0 = (c0 / 600e-9 * 2 * pi);    % resonance frequency
eps0 = 8.85e-12;                % permittivity in vacuum
I_at = 3.5e20;                  % characteristic intensity
E_at = sqrt(2 * I_at / eps0 / c0); % field strength 

% -------------------------------------------------------------------------
% Laser parameters
% -------------------------------------------------------------------------
lambda1 = 800e-9;
w1 = c0 / lambda1 * 2 * pi;

% -------------------------------------------------------------------------
% Time and frequency grids
% -------------------------------------------------------------------------
Nt = 1024 * 64 * 10;
t = linspace(-50e-15, 50e-15, Nt);
dt = t(2) - t(1);
df = 1/Nt/dt;
w = [[0 : Nt/2-1], [-Nt/2 : -1]] * df * 2 * pi;

% -------------------------------------------------------------------------
% Laser field
% -------------------------------------------------------------------------
E = E_at / 100 *exp(-2*log(2)*t.^2 / 10e-15.^2) .* cos(w1*t);

% plot(t, E);
% return;
% -------------------------------------------------------------------------
% Memory allocation
% -------------------------------------------------------------------------
ddx = zeros(1, Nt);             % acceleration
dx = zeros(1, Nt);              % velocity
x = zeros(1, Nt);               % position

% -------------------------------------------------------------------------
% Solve the equation of motion using numerical integration
% -------------------------------------------------------------------------
for tidx = 1 : Nt-1
    x(tidx + 1) = x(tidx) + dt * dx(tidx);
    dx(tidx + 1) = dx(tidx) + dt * ddx(tidx);
    ddx(tidx + 1) = - 2 * gam * dx(tidx+1) - w0^2*x(tidx+1) - a*x(tidx+1).^2 - e * E(tidx + 1) / me;
end

% -------------------------------------------------------------------------
% Plot results
% -------------------------------------------------------------------------
figure(1); clf;
plot(t, E/max(E), 'r', t, x/1e-10, 'b');
xlabel('Time (s)'); ylabel('x (ang.)');
legend({'E', 'x'});